@@ -31,6 +31,9 @@
 #include <ukcprog.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#include <sys/termios.h>
+#include <signal.h>
+#include <errno.h>
 
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
@@ -144,6 +147,11 @@ typedef struct machine_data_t machine_data_list_t;
 
 #define SORTBYMAXNAME 128
 typedef struct{
+	int maxx;
+	int maxy;
+
+	char units;
+
 	int cpu_user;
 	int cpu_idle;
 	int cpu_iowait;
@@ -168,11 +176,7 @@ typedef struct{
 	int pages_in;
 	int pages_out;
 
-	int processes_total;
-	int processes_sleeping;
-	int processes_cpu;
-	int processes_zombie;
-	int processes_stopped;
+	int processes;
 
 	int network_io_total_tx;
 	int network_io_total_rx;
@@ -217,26 +221,7 @@ MKCMP(swap_used_pecent)
 #define DISKIOW "DiskIO writes"
 #define DISKIO "Total DiskIO (reads+writes)"
 
-
-/*
-int cmp_cpu(machine_data_list_t *a, machine_data_list_t *b){
-
-	if(a->cpu_used == b->cpu_used){
-		if(a->load_1 == b->load_1) return 0;
-		if(a->load_1 > b->load_1){
-			return -1;
-		}else{
-			return 1;
-		}
-	}
-
-	if((a->cpu_used) > (b->cpu_used)){
-		return -1;
-	}else{
-		return 1;
-	}
-}
-*/
+int sig_winch=0;
 
 #ifndef HAVE_ATOLL
 long long int atoll (const char *nptr){
@@ -675,109 +660,294 @@ int parse_xml(char *xml, machine_data_list_t **md){
 	
 }
 
-void display(machine_data_list_t *machine_data_list, display_config_t *display_config, int num_lines, int *title){
+void display(machine_data_list_t *machine_data_list, display_config_t *display_config, int *title){
 	int line_num=4;
 	int counter;
 	int x=1;
 
 	if(*title){
 		clear();
-		move (num_lines-3, 1);
+		move (display_config->maxy-3, 1);
 		printw("Sorting by %-64s", display_config->sortby);
+		move (display_config->maxy-2, 1);
+		if(display_config->units == 'b'){
+			printw("Units are measured in bytes/sec");
+		}
+		if(display_config->units == 'k'){
+                        printw("Units are measured in kilobytes/sec");
+                }
+		if(display_config->units == 'm'){
+                        printw("Units are measured in megabytes/sec");
+		}
 
 		move(1,1);
 		printw("%-11s", "Hostname");
 		x=x+11+1;
-		if(display_config->cpu_used){
+		if(display_config->cpu_used && (display_config->maxx > x+6)){
 			move(1,x);
 			printw("%5s", "CPU");
 			move(2,x);
 			printw("%5s", "used%");
 			x+=6;
 		}
-		if(display_config->load_1){
+		if(display_config->load_1 && (display_config->maxx > x+6)){
 			move(1,x);
 			printw("%5s", "Load");
 			move(2,x);
 			printw("%5s", "(1m)");
 			x+=6;
 		}
-		if(display_config->pages_in){
+
+		if(display_config->pages_in && (display_config->maxx > x+6)){
 			move(1,x);
 			printw("%5s", "Page");
 			move(2,x);
 			printw("%5s", "ins");
 			x+=6;
 		}
-		if(display_config->pages_out){
+
+		if(display_config->pages_out && (display_config->maxx > x+6)){
 			move(1,x);
 			printw("%5s", "Page");
 			move(2,x);
                         printw("%5s", "outs");
                         x+=6;
 		}
-		if(display_config->memory_used_pecent){
+
+		if(display_config->memory_used_pecent && (display_config->maxx > x+6)){
 			move(1,x);
 			printw("%5s", "Mem");
 			move(2,x);
                         printw("%5s", "used");
                         x+=6;
 		}
-                if(display_config->swap_used_pecent){
+
+                if(display_config->swap_used_pecent  && (display_config->maxx > x+6)){
 			move(1,x);
                         printw("%5s", "Swap");
 			move(2,x);
                         printw("%5s", "used");
                         x+=6;
                 }
-		if(display_config->network_io_total_rx){
-			move(1,x);
-			printw("%8s", "Net");
-			move(2,x);
-			printw("%8s", "rx");
-			x+=9;
+
+		if(display_config->network_io_total_rx){ 
+			if(display_config->units=='b' && (display_config->maxx > x+9)){
+				move(1,x);
+				printw("%8s", "Net");
+				move(2,x);
+				printw("%8s", "rx");
+				x+=9;
+			}
+
+			if(display_config->units=='k' && (display_config->maxx > x+6)){
+				move(1,x);
+                                printw("%5s", "Net");
+                                move(2,x);
+                                printw("%5s", "rx");
+                                x+=6;
+                        }
+
+			if(display_config->units=='m' && (display_config->maxx > x+6)){
+                                move(1,x);
+                                printw("%5s", "Net");
+                                move(2,x);
+                                printw("%5s", "rx");
+                                x+=6;
+                        }
+
 		}
+
 		if(display_config->network_io_total_tx){
-			move(1,x);
-                        printw("%8s", "Net");
-			move(2,x);
-                        printw("%8s", "tx");
-			x+=9;
+                        if(display_config->units=='b' && (display_config->maxx > x+9)){
+                                move(1,x);
+                                printw("%8s", "Net");
+                                move(2,x);
+                                printw("%8s", "tx");
+                                x+=9;
+                        }
+
+                        if(display_config->units=='k' && (display_config->maxx > x+6)){
+                                move(1,x);
+                                printw("%5s", "Net");
+                                move(2,x);
+                                printw("%5s", "tx");
+                                x+=6;
+                        }
+
+                        if(display_config->units=='m' && (display_config->maxx > x+6)){
+                                move(1,x);
+                                printw("%5s", "Net");
+                                move(2,x);
+                                printw("%5s", "tx");
+                                x+=6;
+                        }
+
                 }
+
 		if(display_config->disk_io_total_read){
-			move(1,x);
-			printw("%9s", "Disk");
-			move(2,x);
-			printw("%9s", "read");
-			x+=10;
+			if(display_config->units=='b' && (display_config->maxx > x+10)){
+				move(1,x);
+				printw("%9s", "Disk");
+				move(2,x);
+				printw("%9s", "read");
+				x+=10;
+			}
+
+			if(display_config->units=='k' && (display_config->maxx > x+7)){
+				move(1,x);
+                                printw("%6s", "Disk");
+                                move(2,x);
+                                printw("%6s", "read");
+                                x+=7;
+                        }
+
+			if(display_config->units=='m' && (display_config->maxx > x+7)){
+                                move(1,x);
+                                printw("%6s", "Disk");
+                                move(2,x);
+                                printw("%6s", "read");
+                                x+=7;
+                        }
+
 		}
+
 		if(display_config->disk_io_total_read){
-			move(1,x);
-                        printw("%9s", "Disk");
-			move(2,x);
-                        printw("%9s", "write");
-			x+=10;
+                        if(display_config->units=='b' && (display_config->maxx > x+10)){
+                                move(1,x);
+                                printw("%9s", "Disk");
+                                move(2,x);
+                                printw("%9s", "write");
+                                x+=10;
+                        }
+
+                        if(display_config->units=='k' && (display_config->maxx > x+7)){
+                                move(1,x);
+                                printw("%6s", "Disk");
+                                move(2,x);
+                                printw("%6s", "write");
+                                x+=7;
+                        }
+
+                        if(display_config->units=='m' && (display_config->maxx > x+7)){
+                                move(1,x);
+                                printw("%6s", "Disk");
+                                move(2,x);
+                                printw("%6s", "write");
+                                x+=7;
+                        }
+
                 }
-		
+
+		if(display_config->processes && (display_config->maxx > x+25)){
+			move(1,x);
+			printw("%-24s", "   Number of Process");
+			move(2,x);
+			printw("%-24s", " Run Slep Zomb Stop  Tot");
+			x+=25;
+		}
+
 		*title=0;
 	}
 
 	/* Counter starts at 8, for padding (eg, headers, borders etc) */
-	for(counter=8;counter<num_lines;counter++){	
+	for(counter=8;counter<display_config->maxy;counter++){	
 		if(machine_data_list==NULL) break;
 		move(line_num++, 1);
 	   	printw("%-11s", machine_data_list->sysname);
+		x=13;
 
-		if(display_config->cpu_used) 		printw(" %5.1f", machine_data_list->cpu_used);
-		if(display_config->load_1) 		printw(" %5.1f", machine_data_list->load_1);
-		if(display_config->pages_in)	 	printw(" %5d", machine_data_list->pages_in);
-		if(display_config->pages_out) 		printw(" %5d", machine_data_list->pages_out);
-		if(display_config->memory_used_pecent) 	printw(" %5.1f", machine_data_list->memory_used_pecent);
-		if(display_config->swap_used_pecent) 	printw(" %5.1f", machine_data_list->swap_used_pecent);
-		if(display_config->network_io_total_rx)	printw(" %8lld", machine_data_list->network_io_total_rx);
-		if(display_config->network_io_total_tx)	printw(" %8lld", machine_data_list->network_io_total_tx);
-		if(display_config->disk_io_total_read)	printw(" %9lld", machine_data_list->disk_io_total_read);
-		if(display_config->disk_io_total_write)	printw(" %9lld", machine_data_list->disk_io_total_write);
+		if(display_config->cpu_used && (display_config->maxx > x+6)){
+			printw(" %5.1f", machine_data_list->cpu_used);
+			x+=6;
+		}
+		if(display_config->load_1 && (display_config->maxx > x+6)){
+			printw(" %5.1f", machine_data_list->load_1);
+			x+=6;
+		}
+		if(display_config->pages_in && (display_config->maxx > x+6)){
+			printw(" %5d", machine_data_list->pages_in);
+			x+=6;
+		}
+		if(display_config->pages_out && (display_config->maxx > x+6)){
+			printw(" %5d", machine_data_list->pages_out);
+			x+=6;
+		}
+		if(display_config->memory_used_pecent && (display_config->maxx > x+6)){
+			printw(" %5.1f", machine_data_list->memory_used_pecent);
+			x+=6;
+		}
+		if(display_config->swap_used_pecent && (display_config->maxx > x+6)){
+			printw(" %5.1f", machine_data_list->swap_used_pecent);
+			x+=6;
+                }
+
+		if(display_config->network_io_total_rx){
+			if(display_config->units=='b' && (display_config->maxx > x+9)){
+				printw(" %8lld", machine_data_list->network_io_total_rx);
+				x+=9;
+			}
+			if(display_config->units=='k' && (display_config->maxx > x+6)){
+                                printw(" %5lld", machine_data_list->network_io_total_rx/1024);
+                                x+=6;
+                        }
+                        if(display_config->units=='m' && (display_config->maxx > x+6)){
+                                printw(" %5.2f", (double)(machine_data_list->network_io_total_rx/(1024.00*1024.00)));
+                                x+=6;
+                        }
+		}
+
+		if(display_config->network_io_total_tx){
+                        if(display_config->units=='b' && (display_config->maxx > x+9)){
+                                printw(" %8lld", machine_data_list->network_io_total_tx);
+                                x+=9;
+                        }
+                        if(display_config->units=='k' && (display_config->maxx > x+6)){
+                                printw(" %5lld", machine_data_list->network_io_total_tx/1024);
+                                x+=6;
+                        }
+                        if(display_config->units=='m' && (display_config->maxx > x+6)){
+                                printw(" %5.2f", (double)(machine_data_list->network_io_total_tx/(1024.00*1024.00)));
+                                x+=6;
+                        }
+		}
+
+		if(display_config->disk_io_total_read){
+			if(display_config->units=='b' && (display_config->maxx > x+10)){
+				printw(" %9lld", machine_data_list->disk_io_total_read);
+				x+=10;
+			}
+			if(display_config->units=='k' && (display_config->maxx > x+7)){
+                                printw(" %6lld", machine_data_list->disk_io_total_read/1024);
+                                x+=7;
+			}
+			if(display_config->units=='m' && (display_config->maxx > x+7)){
+                                printw(" %6.2f", (double)(machine_data_list->disk_io_total_read/(1024.00*1024.00)));
+                                x+=7;
+                        }
+		}
+
+		if(display_config->disk_io_total_write){
+                        if(display_config->units=='b' && (display_config->maxx > x+10)){
+                                printw(" %9lld", machine_data_list->disk_io_total_write);
+                                x+=10;
+                        }
+                        if(display_config->units=='k' && (display_config->maxx > x+7)){
+                                printw(" %6lld", machine_data_list->disk_io_total_write/1024);
+                                x+=7;
+                        }
+                        if(display_config->units=='m' && (display_config->maxx > x+7)){
+                                printw(" %6.2f", (double)(machine_data_list->disk_io_total_write/(1024.00*1024.00)));
+                                x+=7;
+                       	}
+		}
+		if(display_config->processes && display_config->maxx > x+25){
+			printw(" %4d %4d %4d %4d %4d", machine_data_list->processes_cpu, \
+				machine_data_list->processes_sleeping, \
+				machine_data_list->processes_zombie, \
+				machine_data_list->processes_stopped, \
+				machine_data_list->processes_total);
+			x+=25;
+		}
 
 		machine_data_list=machine_data_list->next;
 	}
@@ -787,10 +957,16 @@ void display(machine_data_list_t *machine_data_list, display_config_t *display_c
 	
 }
 
+void sig_winch_handler(int sig){
+
+	sig_winch=1;	
+	signal(SIGWINCH, sig_winch_handler);
+}
+
 int main(int argc, char **argv){
 	WINDOW *window;
 	fd_set infds;
-	int maxx, maxy;
+	struct winsize size;
 
 	FILE *control;
 	FILE *data;
@@ -805,12 +981,13 @@ int main(int argc, char **argv){
 	machine_data_list_t *machine_data_list=NULL;
 
 	int num_hosts;
-	int max_display=0;
 	int title=1;
 
 	int cmdopt;
         extern int optind;
 	extern char *optarg;
+
+	extern int errno;
 	
 	display_config_t display_config;
 	char ch;
@@ -820,6 +997,8 @@ int main(int argc, char **argv){
 	sortby_ptr=NULL;
 
 	/* What to display defaults */
+	display_config.units='b';
+	
 	display_config.cpu_user=0;
 	display_config.cpu_idle=0;
 	display_config.cpu_iowait=0;
@@ -844,11 +1023,7 @@ int main(int argc, char **argv){
 	display_config.pages_in=1;
 	display_config.pages_out=1;
 
-	display_config.processes_total=0;
-	display_config.processes_sleeping=0;
-	display_config.processes_cpu=0;
-	display_config.processes_zombie=0;
-	display_config.processes_stopped=0;
+	display_config.processes=1;
 
 	display_config.network_io_total_tx=1;
 	display_config.network_io_total_rx=1;
@@ -859,13 +1034,12 @@ int main(int argc, char **argv){
 	display_config.disk_io_all_stats=0;
 
 	display_config.disk_total_used=0;
-	display_config.disk_all_stats=0; 
+	display_config.disk_all_stats=0;
 
-	while((cmdopt=getopt(argc, argv, "d:s:"))  != -1){
+	signal(SIGWINCH, sig_winch_handler); 
+
+	while((cmdopt=getopt(argc, argv, "s:"))  != -1){
                 switch(cmdopt){
-			case 'd':
-				max_display=atoi(optarg);
-			break;
 			case 's':
 				if(!strcmp(optarg, "cpu")){
 					sortby_ptr=cmp_cpu_used;
@@ -907,6 +1081,7 @@ int main(int argc, char **argv){
 	control=create_tcp_connection(servername, server_control_port);
 	if(control==NULL){
 		errf("Failed to connect (%m)");
+		exit(1);
 	}
 
 	if(argc==4){
@@ -950,20 +1125,15 @@ int main(int argc, char **argv){
 	data=create_tcp_connection(servername, server_data_port);
 	if(data==NULL){
 		errf("Failed to connect to host %s on port %d (%m)",servername, server_data_port);
+		exit(1);
 	}
 
-	/*	
-	printf("\033[2J");
-	printf("\033[1;1HHostname     CPU   Load  Page  Page  Mem   Swap      Net      Net      Disk     Disk");
-	printf("\033[2;1H             used%% (1m)  ins   outs  used  used      rx       tx       read     write");	
-	*/
-
 	initscr();
 	nonl();
 	cbreak();
-	echo();
+	noecho();
 	window=newwin(0, 0, 0, 0);
-	getmaxyx(window, maxy, maxx);
+	getmaxyx(window, display_config.maxy, display_config.maxx);
 
 	stdin_fileno=fileno(stdin);
 	data_fileno=fileno(data);
@@ -973,12 +1143,33 @@ int main(int argc, char **argv){
 		FD_ZERO(&infds);
 		FD_SET(stdin_fileno, &infds);
 		FD_SET(data_fileno, &infds);
-		select(biggest_fileno, &infds, NULL, NULL, NULL);
+		if((select(biggest_fileno, &infds, NULL, NULL, NULL))==-1){
+			if(errno!=EINTR){
+				errf("select failed with (%m)");
+				exit(1);
+			}
+		}
+
+		if(sig_winch){
+			if (ioctl(fileno(stdout), TIOCGWINSZ, &size) == 0) {
+	                	resizeterm(size.ws_row, size.ws_col);
+                        	wrefresh(curscr);
+			}
+			getmaxyx(window, display_config.maxy, display_config.maxx);
+			title=1;
+			display(machine_data_list, &display_config, &title);
+			refresh();
+			sig_winch=0;
+			continue;
+		}
 		
 		if(FD_ISSET(stdin_fileno, &infds)){
 		
-			ch=getc(stdin);
+			ch=getch();
 			switch(ch){
+				case KEY_RESIZE:
+					sig_winch=1;
+					break;						
 
 				/* Quit */
 				case 'Q':
@@ -986,6 +1177,17 @@ int main(int argc, char **argv){
 					endwin();
 					exit(0);
 					break;
+				/* Units */
+				case 'U':
+				case 'u':
+					if(display_config.units == 'b'){
+						display_config.units = 'k';
+					}else if(display_config.units == 'k'){
+						display_config.units = 'm';
+					}else{
+						display_config.units = 'b';
+					}
+					break;
 
 				/* Sort by */
 				case 'C':
@@ -1091,13 +1293,16 @@ int main(int argc, char **argv){
 						display_config.cpu_used=1;
 					}
 					break;
+				case 'r':
+					if(display_config.processes){
+						display_config.processes=0;
+					}else{
+						display_config.processes=1;
+					}
+					break;
 
 				default:
-					/* Invalid key.. Ignore.. Set Title to -1, as the
-					 * title++ will then make that "0" (false) so a 
-					 * screen redraw will not happen */
-					title=-1;
-					break;
+					continue;
 			}
 
 			/* Increment title so it becomes true (and making the screen update */	
@@ -1116,11 +1321,7 @@ int main(int argc, char **argv){
 		num_hosts=parse_xml(response, &machine_data_list);
 		if(num_hosts==-1) continue;
 		machine_data_list=sort_machine_stats(machine_data_list, num_hosts, sortby_ptr);
-		if(max_display==0){
-			display(machine_data_list, &display_config, maxy, &title);
-		}else{
-			display(machine_data_list, &display_config, max_display, &title);
-		}
+		display(machine_data_list, &display_config, &title);
 
 	}
 	exit(0);
